@@ -3,3 +3,7 @@ Console module, contains the console entry point and different subcommands.
 """
 
 from spotdl.console.entry_point import console_entry_point
+
+__all__ = [
+    "console_entry_point",
+]
