@@ -3,11 +3,13 @@ Playlist module for retrieving playlist data from Spotify.
 """
 
 from dataclasses import dataclass
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Tuple
 
-from spotdl.types.song import SongList
+from spotdl.types.song import Song, SongList
 from spotdl.utils.spotify import SpotifyClient
 
+__all__ = ["Playlist", "PlaylistError"]
+
 
 class PlaylistError(Exception):
     """
@@ -26,71 +28,8 @@ class Playlist(SongList):
     author_name: str
     cover_url: str
 
-    @classmethod
-    def search(cls, search_term: str):
-        """
-        Searches for Playlist from a search term.
-
-        ### Arguments
-        - search_term: The search term to use.
-
-        ### Returns
-        - The raw search results
-        """
-
-        spotify_client = SpotifyClient()
-        raw_search_results = spotify_client.search(search_term, type="playlist")
-
-        if (
-            raw_search_results is None
-            or len(raw_search_results.get("playlists", {}).get("items", [])) == 0
-        ):
-            raise PlaylistError("No playlist matches found on spotify")
-
-        return raw_search_results
-
     @staticmethod
-    def get_urls(url: str) -> List[str]:
-        """
-        Get URLs of all tracks in a playlist.
-
-        ### Arguments
-        - url: The URL of the playlist.
-
-        ### Returns
-        - A list of urls.
-        """
-
-        spotify_client = SpotifyClient()
-        tracks = []
-
-        playlist_response = spotify_client.playlist_items(url)
-        if playlist_response is None:
-            raise PlaylistError(f"Wrong playlist id: {url}")
-
-        tracks = playlist_response["items"]
-
-        # Get all tracks from playlist
-        while playlist_response["next"]:
-            playlist_response = spotify_client.next(playlist_response)
-
-            # Failed to get response, break the loop
-            if playlist_response is None:
-                break
-
-            # Add tracks to the list
-            tracks.extend(playlist_response["items"])
-
-        return [
-            track["track"]["external_urls"]["spotify"]
-            for track in tracks
-            if track is not None
-            and track.get("track") is not None
-            and track.get("track").get("id")
-        ]
-
-    @staticmethod
-    def get_metadata(url: str) -> Dict[str, Any]:
+    def get_metadata(url: str) -> Tuple[Dict[str, Any], List[Song]]:
         """
         Get metadata for a playlist.
 
@@ -107,7 +46,7 @@ class Playlist(SongList):
         if playlist is None:
             raise PlaylistError("Invalid playlist URL.")
 
-        return {
+        metadata = {
             "name": playlist["name"],
             "url": url,
             "description": playlist["description"],
@@ -124,3 +63,65 @@ class Playlist(SongList):
                 else ""
             ),
         }
+
+        playlist_response = spotify_client.playlist_items(url)
+        if playlist_response is None:
+            raise PlaylistError(f"Wrong playlist id: {url}")
+
+        # Get all tracks from playlist
+        tracks = playlist_response["items"]
+        while playlist_response["next"]:
+            playlist_response = spotify_client.next(playlist_response)
+
+            # Failed to get response, break the loop
+            if playlist_response is None:
+                break
+
+            # Add tracks to the list
+            tracks.extend(playlist_response["items"])
+
+        songs = []
+        for track in tracks:
+            if (
+                not isinstance(track, dict)
+                or track.get("track") is None
+                or track.get("track", {}).get("is_local")
+            ):
+                continue
+
+            track_meta = track.get("track", {})
+            track_id = track_meta.get("id")
+            if track_id is None or track_meta.get("duration_ms") == 0:
+                continue
+
+            album_meta = track_meta["album"]
+            release_date = album_meta["release_date"]
+
+            artists = [artist["name"] for artist in track_meta["artists"]]
+            song = Song.from_missing_data(
+                name=track_meta["name"],
+                artists=artists,
+                artist=artists[0],
+                album_id=album_meta["id"],
+                album_name=album_meta["name"],
+                album_artist=album_meta["artists"][0]["name"],
+                disc_number=track_meta["disc_number"],
+                duration=track_meta["duration_ms"],
+                year=release_date[:4],
+                date=release_date,
+                track_number=track_meta["track_number"],
+                tracks_count=album_meta["total_tracks"],
+                song_id=track_meta["id"],
+                explicit=track_meta["explicit"],
+                url=track_meta["external_urls"]["spotify"],
+                isrc=track_meta.get("external_ids", {}).get("isrc"),
+                cover_url=max(
+                    album_meta["images"], key=lambda i: i["width"] * i["height"]
+                )["url"]
+                if (len(album_meta["images"]) > 0)
+                else None,
+            )
+
+            songs.append(song)
+
+        return metadata, songs
