@@ -5,21 +5,18 @@ Module that handles the command line arguments.
 import argparse
 import sys
 import textwrap
-
-from argparse import _ArgumentGroup, ArgumentParser, Namespace
+from argparse import ArgumentParser, Namespace, _ArgumentGroup
 from typing import List
 
 from spotdl import _version
-from spotdl.download.progress_handler import NAME_TO_LEVEL
+from spotdl.download.downloader import AUDIO_PROVIDERS, LYRICS_PROVIDERS
 from spotdl.utils.ffmpeg import FFMPEG_FORMATS
 from spotdl.utils.formatter import VARS
-from spotdl.download.downloader import (
-    AUDIO_PROVIDERS,
-    LYRICS_PROVIDERS,
-)
+from spotdl.utils.logging import NAME_TO_LEVEL
 
+__all__ = ["OPERATIONS", "SmartFormatter", "parse_arguments"]
 
-OPERATIONS = ["download", "save", "web", "sync", "meta"]
+OPERATIONS = ["download", "save", "web", "sync", "meta", "url"]
 
 
 class SmartFormatter(argparse.HelpFormatter):
@@ -41,56 +38,6 @@ class SmartFormatter(argparse.HelpFormatter):
         return textwrap.wrap(text, width)
 
 
-def parse_arguments() -> Namespace:
-    """
-    Parse arguments from the command line.
-
-    ### Returns
-    - A Namespace object containing the parsed arguments.
-    """
-
-    # Initialize argument parser
-    parser = ArgumentParser(
-        prog="spotdl",
-        description="Download your Spotify playlists and songs along with album art and metadata",
-        formatter_class=SmartFormatter,
-        epilog=(
-            "For more information, visit http://spotdl.rtfd.io/ "
-            "or join our Discord server: https://discord.com/invite/xCa23pwJWY"
-        ),
-    )
-
-    # Parse main options
-    main_options = parser.add_argument_group("Main options")
-    parse_main_options(main_options)
-
-    # Parse spotify options
-    spotify_options = parser.add_argument_group("Spotify options")
-    parse_spotify_options(spotify_options)
-
-    # Parse ffmpeg options
-    ffmpeg_options = parser.add_argument_group("FFmpeg options")
-    parse_ffmpeg_options(ffmpeg_options)
-
-    # Parse output options
-    output_options = parser.add_argument_group("Output options")
-    parse_output_options(output_options)
-
-    # Parse web options
-    web_options = parser.add_argument_group("Web options")
-    parse_web_options(web_options)
-
-    # Parse misc options
-    misc_options = parser.add_argument_group("Misc options")
-    parse_misc_options(misc_options)
-
-    # Parse other options
-    other_options = parser.add_argument_group("Other options")
-    parse_other_options(other_options)
-
-    return parser.parse_args()
-
-
 def parse_main_options(parser: _ArgumentGroup):
     """
     Parse main options from the command line.
@@ -113,6 +60,7 @@ def parse_main_options(parser: _ArgumentGroup):
             "web: Starts a web interface to simplify the download process.\n"
             "sync: Removes songs that are no longer present, downloads new ones\n"
             "meta: Update your audio files with metadata\n"
+            "url: Get the download URL for songs\n\n"
         ),
     )
 
@@ -122,10 +70,14 @@ def parse_main_options(parser: _ArgumentGroup):
         nargs="+",
         type=str,
         help=(
-            "Spotify URL for a song/playlist/album/artist/etc. to download. "
-            "For album searching, include 'album:' and optional 'artist:' tags "
-            "(ie. 'album:the album name' or 'artist:the artist album: the album'). "
-            "For manual audio matching, you can use the format 'YouTubeURL|SpotifyURL'"
+            "N|Spotify/YouTube URL for a song/playlist/album/artist/etc. to download.\n"
+            "For album searching, include 'album:' and optional 'artist:' tags\n"
+            "(ie. 'album:the album name' or 'artist:the artist album: the album').\n"
+            "For manual audio matching, you can use the format 'YouTubeURL|SpotifyURL'\n"
+            "You can only use album/playlist/tracks urls when "
+            "downloading/matching youtube urls.\n"
+            "When using youtube url without spotify url, "
+            "you won't be able to use `--fetch-albums` option.\n\n"
         ),
     )
 
@@ -181,6 +133,7 @@ def parse_main_options(parser: _ArgumentGroup):
     parser.add_argument(
         "--search-query",
         help=f"The search query to use, available variables: {', '.join(VARS)}",
+        type=str,
     )
 
     # Add don't filter results argument
@@ -213,18 +166,21 @@ def parse_spotify_options(parser: _ArgumentGroup):
     parser.add_argument(
         "--client-id",
         help="The client id to use when logging in to Spotify.",
+        type=str,
     )
 
     # Add client secret argument
     parser.add_argument(
         "--client-secret",
         help="The client secret to use when logging in to Spotify.",
+        type=str,
     )
 
     # Add auth token argument
     parser.add_argument(
         "--auth-token",
-        help="The authorisation token to use directly to log in to Spotify.",
+        help="The authorization token to use directly to log in to Spotify.",
+        type=str,
     )
 
     # Add cache path argument
@@ -242,10 +198,19 @@ def parse_spotify_options(parser: _ArgumentGroup):
         help="Disable caching (both requests and token).",
     )
 
-    # Add cookie file argument
+    # Add max retries argument
     parser.add_argument(
-        "--cookie-file",
-        help="Path to cookies file.",
+        "--max-retries",
+        type=int,
+        help="The maximum number of retries to perform when getting metadata.",
+    )
+
+    # Add headless argument
+    parser.add_argument(
+        "--headless",
+        action="store_const",
+        const=True,
+        help="Run in headless mode.",
     )
 
 
@@ -261,6 +226,7 @@ def parse_ffmpeg_options(parser: _ArgumentGroup):
     parser.add_argument(
         "--ffmpeg",
         help="The ffmpeg executable to use.",
+        type=str,
     )
 
     # Add search threads argument
@@ -269,10 +235,13 @@ def parse_ffmpeg_options(parser: _ArgumentGroup):
         type=int,
         help="The number of threads to use when downloading songs.",
     )
+
     # Add constant bit rate argument
     parser.add_argument(
         "--bitrate",
         choices=[
+            "auto",
+            "disable",
             "8k",
             "16k",
             "24k",
@@ -295,6 +264,9 @@ def parse_ffmpeg_options(parser: _ArgumentGroup):
         help=(
             "The constant/variable bitrate to use for the output file. "
             "Values from 0 to 9 are variable bitrates. "
+            "Auto will use the bitrate of the original file. "
+            "Disable will disable the bitrate option. "
+            "(In case of m4a and opus files, auto and disable will skip the conversion)"
         ),
     )
 
@@ -305,18 +277,6 @@ def parse_ffmpeg_options(parser: _ArgumentGroup):
         help="Additional ffmpeg arguments passed as a string.",
     )
 
-    # Preserve original audio stream
-    parser.add_argument(
-        "--preserve-original-audio",
-        action="store_const",
-        const=True,
-        help=(
-            "Preserve the original audio stream in case of m4a and opus files. "
-            "This option might overwrite the bitrate option. "
-            "Adding additional ffmpeg arguments might make this option useless."
-        ),
-    )
-
 
 def parse_output_options(parser: _ArgumentGroup):
     """
@@ -331,6 +291,7 @@ def parse_output_options(parser: _ArgumentGroup):
         "--format",
         choices=FFMPEG_FORMATS.keys(),
         help="The format to download the song in.",
+        type=str,
     )
 
     # Add save file argument
@@ -369,7 +330,7 @@ def parse_output_options(parser: _ArgumentGroup):
         help=(
             "Name of the m3u file to save the songs to. "
             "Defaults to {list[0]}.m3u "
-            "If you want to generate a m3u for each list in the query use {list-name}, "
+            "If you want to generate a m3u for each list in the query use {list}, "
             "If you want to generate a m3u file based on the first list in the query use {list[0]}"
             ", (0 is the first list in the query, 1 is the second, etc. "
             "songs don't count towards the list number) "
@@ -377,11 +338,24 @@ def parse_output_options(parser: _ArgumentGroup):
         const="{list[0]}.m3u",
     )
 
+    # Add cookie file argument
+    parser.add_argument(
+        "--cookie-file",
+        help="Path to cookies file.",
+        type=str,
+    )
+
     # Add overwrite argument
     parser.add_argument(
         "--overwrite",
         choices={"force", "skip", "metadata"},
-        help="Overwrite existing files.",
+        help=(
+            "How to handle existing/duplicate files. "
+            "(When combined with --scan-for-songs force will remove "
+            "all duplicates, and metadata will only apply metadata to the "
+            "latest song and will remove the rest. )"
+        ),
+        type=str,
     )
 
     # Option to restrict filenames for easier handling in the shell
@@ -426,6 +400,51 @@ def parse_output_options(parser: _ArgumentGroup):
             and album art as the playlist's icon",
     )
 
+    # Option to scan the output directory for existing files
+    parser.add_argument(
+        "--scan-for-songs",
+        action="store_const",
+        const=True,
+        help=(
+            "Scan the output directory for existing files. "
+            "This option should be combined with the --overwrite option "
+            "to control how existing files are handled. (Output directory is the last directory "
+            "that is not a template variable in the output template)"
+        ),
+    )
+
+    # Option to fetch all albums from songs in query
+    parser.add_argument(
+        "--fetch-albums",
+        action="store_const",
+        const=True,
+        help="Fetch all albums from songs in query",
+    )
+
+    # Option to change the id3 separator
+    parser.add_argument(
+        "--id3-separator",
+        type=str,
+        help="Change the separator used in the id3 tags. Only supported for mp3 files.",
+    )
+
+    # Option to use ytm data instead of spotify data
+    # when downloading using ytm link
+    parser.add_argument(
+        "--ytm-data",
+        action="store_const",
+        const=True,
+        help="Use ytm data instead of spotify data when downloading using ytm link.",
+    )
+
+    # Option whether to add unavailable songs to the m3u file
+    parser.add_argument(
+        "--add-unavailable",
+        action="store_const",
+        const=True,
+        help="Add unavailable songs to the m3u/archive files when downloading",
+    )
+
 
 def parse_web_options(parser: _ArgumentGroup):
     """
@@ -464,6 +483,25 @@ def parse_web_options(parser: _ArgumentGroup):
         help="The allowed origins for the web server.",
     )
 
+    # Add use output directory argument
+    parser.add_argument(
+        "--web-use-output-dir",
+        action="store_const",
+        const=True,
+        help=(
+            "Use the output directory instead of the session directory for downloads. ("
+            "This might cause issues if you have multiple users using the web-ui at the same time)"
+        ),
+    )
+
+    # Add keep sessions argument
+    parser.add_argument(
+        "--keep-sessions",
+        action="store_const",
+        const=True,
+        help="Keep the session directory after the web server is closed.",
+    )
+
 
 def parse_misc_options(parser: _ArgumentGroup):
     """
@@ -488,14 +526,6 @@ def parse_misc_options(parser: _ArgumentGroup):
         help="Use a simple tui.",
     )
 
-    # Add headless argument
-    parser.add_argument(
-        "--headless",
-        action="store_const",
-        const=True,
-        help="Run in headless mode.",
-    )
-
 
 def parse_other_options(parser: _ArgumentGroup):
     """
@@ -534,3 +564,71 @@ def parse_other_options(parser: _ArgumentGroup):
         help="Show the version number and exit.",
         version=_version.__version__,
     )
+
+
+def create_parser() -> ArgumentParser:
+    """
+    Parse arguments from the command line.
+
+    ### Returns
+    - A Namespace object containing the parsed arguments.
+    """
+
+    # Initialize argument parser
+    parser = ArgumentParser(
+        prog="spotdl",
+        description="Download your Spotify playlists and songs along with album art and metadata",
+        formatter_class=SmartFormatter,
+        epilog=(
+            "For more information, visit http://spotdl.rtfd.io/ "
+            "or join our Discord server: https://discord.com/invite/xCa23pwJWY"
+        ),
+    )
+
+    # Parse main options
+    main_options = parser.add_argument_group("Main options")
+    parse_main_options(main_options)
+
+    # Parse spotify options
+    spotify_options = parser.add_argument_group("Spotify options")
+    parse_spotify_options(spotify_options)
+
+    # Parse ffmpeg options
+    ffmpeg_options = parser.add_argument_group("FFmpeg options")
+    parse_ffmpeg_options(ffmpeg_options)
+
+    # Parse output options
+    output_options = parser.add_argument_group("Output options")
+    parse_output_options(output_options)
+
+    # Parse web options
+    web_options = parser.add_argument_group("Web options")
+    parse_web_options(web_options)
+
+    # Parse misc options
+    misc_options = parser.add_argument_group("Misc options")
+    parse_misc_options(misc_options)
+
+    # Parse other options
+    other_options = parser.add_argument_group("Other options")
+    parse_other_options(other_options)
+
+    return parser
+
+
+def parse_arguments() -> Namespace:
+    """
+    Parse arguments from the command line.
+
+    ### Arguments
+    - parser: The argument parser to parse the arguments from.
+
+    ### Returns
+    - A Namespace object containing the parsed arguments.
+    """
+
+    # Create parser
+    parser = create_parser()
+
+    # Parse arguments
+    return parser.parse_args()
